import '../base_document.dart';
import 'permission_mode.dart';

/// Class representing a CosmosDB permission.
class CosmosDbPermission extends BaseDocument with EtagMixin {
  CosmosDbPermission(this.id, this.mode, this.resource, [this.token]);

  @override
  final String id;

  /// The permission's mode (see [PermissionMode]).
  final PermissionMode mode;

  /// The resource covered by this permission.
  final String resource;

  /// The permission's token generated by CosmosDB.
  final String? token;

  @override
  Map<String, dynamic> toJson() => {
        'id': id,
        'permissionMode': mode.name,
        'resource': resource,
      };

  /// Builds a [CosmosDbPermission] from a CosmosDB JSON object.
  static CosmosDbPermission build(Map json) {
    final permission = CosmosDbPermission(
        json['id'],
        PermissionMode.parse(json['permissionMode']),
        json['resource'],
        json['_token']);
    permission.setEtag(json);
    return permission;
  }
}
